#include <time.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

uint64_t clk_mono() {
  struct timespec ts;
  clock_gettime(CLOCK_MONOTONIC, &ts);
  return ts.tv_sec * 1000000000L + ts.tv_nsec;
}

static inline uint64_t RDTSC() {
  unsigned hi, lo;
  __asm__ volatile("rdtsc"
                   : "=a"(lo), "=d"(hi)
                   : "a"(0)
                   : "%ebx", "%ecx");
  return ((uint64_t)hi << 32) | lo;
}

#define NS_PER_S 1000000000UL

void clk_test(unsigned sleep_ms) {
  uint64_t mono, mono0, last_mono, tsc;
  mono0 = last_mono = clk_mono();
  printf("# Monotonic_clock TSC\n");
  while (last_mono - mono0 < 60 * NS_PER_S) {
    usleep(1000 * sleep_ms);
    tsc = RDTSC();
    mono = clk_mono();
    if (mono - last_mono >= NS_PER_S / 2)
    {
      printf("%lu %lu\n", mono, tsc);
      last_mono = mono;
    }
  }
}

int main() {
  clk_test(0);
  printf("\n\n");
  clk_test(1);
  printf("\n\n");
  clk_test(10);
  printf("\n\n");
  clk_test(100);
  return 0;
}
